---------------------------------------------------------------------
BEGIN
   DROP TABLE parent;
   EXCEPTION WHEN OTHERS THEN
END;

CREATE TABLE parent (
   p1            VARCHAR ( 10 ) NOT NULL,
   data          INTEGER NOT NULL DEFAULT 0,
   r1            BIGINT NOT NULL DEFAULT AUTOINCREMENT,
   last_updated  TIMESTAMP NOT NULL DEFAULT TIMESTAMP,
   PRIMARY KEY ( p1 ),
   UNIQUE ( r1 ) );

---------------------------------------------------------------------
BEGIN
   DROP TABLE child;
   EXCEPTION WHEN OTHERS THEN
END;

CREATE TABLE child (
   p1            VARCHAR ( 10 ) NOT NULL,
   p2            VARCHAR ( 10 ) NOT NULL,
   data          INTEGER NOT NULL DEFAULT 0,
   r1            BIGINT NOT NULL,
   r2            BIGINT NOT NULL DEFAULT AUTOINCREMENT,
   last_updated  TIMESTAMP NOT NULL DEFAULT TIMESTAMP,
   PRIMARY KEY ( p1, p2 ),
   UNIQUE ( r1, r2 ),
   FOREIGN KEY ( p1 ) 
      REFERENCES parent ( p1 ) ON UPDATE CASCADE );

---------------------------------------------------------------------
BEGIN
   DROP TABLE grandchild1;
   EXCEPTION WHEN OTHERS THEN
END;

CREATE TABLE grandchild1 (
   p1            VARCHAR ( 10 ) NOT NULL,
   p2            VARCHAR ( 10 ) NOT NULL,
   p31           VARCHAR ( 10 ) NOT NULL,
   data          INTEGER NOT NULL DEFAULT 0,
   r1            BIGINT NOT NULL,
   r2            BIGINT NOT NULL,
   r31           BIGINT NOT NULL DEFAULT AUTOINCREMENT,
   last_updated  TIMESTAMP NOT NULL DEFAULT TIMESTAMP,
   PRIMARY KEY ( p1, p2, p31 ),
   UNIQUE ( r1, r2, r31 ),
   FOREIGN KEY ( p1, p2 ) 
      REFERENCES child ( p1, p2 ) ON UPDATE CASCADE );

---------------------------------------------------------------------
BEGIN
   DROP TABLE grandchild2;
   EXCEPTION WHEN OTHERS THEN
END;

CREATE TABLE grandchild2 (
   p1            VARCHAR ( 10 ) NOT NULL,
   p2            VARCHAR ( 10 ) NOT NULL,
   p32           VARCHAR ( 10 ) NOT NULL,
   data          INTEGER NOT NULL DEFAULT 0,
   r1            BIGINT NOT NULL,
   r2            BIGINT NOT NULL,
   r32           BIGINT NOT NULL DEFAULT AUTOINCREMENT,
   last_updated  TIMESTAMP NOT NULL DEFAULT TIMESTAMP,
   PRIMARY KEY ( p1, p2, p32 ),
   UNIQUE ( r1, r2, r32 ),
   FOREIGN KEY ( p1, p2 ) 
      REFERENCES child ( p1, p2 ) ON UPDATE CASCADE );

--------------------------------------------------------------------- 
CALL ml_add_table_script  ( 'v1', 'parent', 'upload_insert', NULL );
CALL ml_add_table_script  ( 'v1', 'parent', 'upload_insert', '
INSERT parent
       ( p1,
         data,
         r1 )
VALUES ( {ml r.p1},
         {ml r.data},
         {ml r.r1} )' );

--------------------------------------------------------------------- 
CALL ml_add_table_script  ( 'v1', 'parent', 'upload_update', NULL );
CALL ml_add_table_script  ( 'v1', 'parent', 'upload_update', '
UPDATE parent
   SET data = {ml r.data}
 WHERE r1 = {ml r.r1}' );

--------------------------------------------------------------------- 
CALL ml_add_table_script  ( 'v1', 'parent', 'upload_delete', NULL );
CALL ml_add_table_script  ( 'v1', 'parent', 'upload_delete', '
DELETE parent
 WHERE r1 = {ml r.r1}' );

--------------------------------------------------------------------- 
CALL ml_add_table_script  ( 'v1', 'parent', 'download_cursor', NULL );
CALL ml_add_table_script  ( 'v1', 'parent', 'download_cursor', '
SELECT p1,
       data,
       r1
  FROM parent
 WHERE last_updated >= {ml s.last_table_download}' );

--------------------------------------------------------------------- 
CALL ml_add_table_script  ( 'v1', 'parent', 'download_delete_cursor', NULL );
CALL ml_add_table_script  ( 'v1', 'parent', 'download_delete_cursor', '--{ml_ignore}' );

--------------------------------------------------------------------- 
-- Insert rows...           p1   p2   p3x   data  r1       r2       r3       last_updated
INSERT parent      VALUES ( 'a',            0,    DEFAULT,                   DEFAULT      );
INSERT child       VALUES ( 'a', 'b',       0,    1,       DEFAULT,          DEFAULT      );
INSERT grandchild1 VALUES ( 'a', 'b', 'c',  0,    1,       1,       DEFAULT, DEFAULT      );
INSERT grandchild2 VALUES ( 'a', 'b', 'c',  0,    1,       1,       DEFAULT, DEFAULT      );
COMMIT;

